@@ -9,6 +9,7 @@ from archimedes.data.common import get_api_base_url_v2
 from archimedes.data.types import PredictionData
 from archimedes.utils.api_request import api
 from archimedes.utils.date import datetime_to_iso_format
+from archimedes.utils.idempotency import generate_idempotency_key
 from archimedes.utils.threaded_executor import execute_many
 
 
@@ -200,6 +201,7 @@ def store_prediction(
     run_dt: Union[str, pd.Timestamp, datetime],
     data: Dict,
     *,
+    use_idempotency: bool = True,
     access_token: str = None,
     **kwargs,
 ) -> bool:
@@ -209,6 +211,7 @@ def store_prediction(
         >>> import archimedes
         >>> import pandas as pd
         >>> from dateutil.tz import gettz
+        >>>
         >>> tz_oslo = gettz("Europe/Oslo")
         >>> d = {
         >>>     "direction": "D",
@@ -233,11 +236,16 @@ def store_prediction(
         "data": data,
     }
 
+    headers = {"Content-Type": "application/json"}
+
+    if use_idempotency:
+        headers["X-Idempotent-Key"] = generate_idempotency_key()
+
     ret = api.request(
         f"{get_api_base_url_v2()}/data/store_prediction",
         method="POST",
         data=json.dumps(payload),
-        headers={"Content-Type": "application/json"},
+        headers=headers,
         access_token=access_token,
         **kwargs,
     )
@@ -249,6 +257,7 @@ def store_predictions(
     prediction_id: str,
     prediction_data: List[PredictionData],
     *,
+    use_idempotency: bool = True,
     access_token: str = None,
     **kwargs,
 ) -> bool:
@@ -258,6 +267,8 @@ def store_predictions(
         >>> import archimedes
         >>> import pandas as pd
         >>> from dateutil.tz import gettz
+        >>> from archimedes.data.types import PredictionData
+        >>>
         >>> tz_oslo = gettz("Europe/Oslo")
         >>> pid = "test-prediction-id"
         >>> from_dt = pd.Timestamp("2021-04-11 23:47:16.854775807", tz=tz_oslo)
@@ -270,7 +281,7 @@ def store_predictions(
         >>>     "price_area": "NO1"
         >>> }
         >>> p_data = [
-        >>>    {'from_dt': from_dt, 'ref_dt': ref_dt, 'run_dt': run_dt, 'data': data }
+        >>>    PredictionData(from_dt=from_dt, ref_dt=ref_dt, run_dt=run_dt, data=data)
         >>> ]
         >>> archimedes.store_predictions(
         >>>     prediction_id=pid,
@@ -294,11 +305,16 @@ def store_predictions(
         "data": predictions,
     }
 
+    headers = {"Content-Type": "application/json"}
+
+    if use_idempotency:
+        headers["X-Idempotent-Key"] = generate_idempotency_key()
+
     ret = api.request(
         f"{get_api_base_url_v2()}/data/store_predictions",
         method="POST",
         data=json.dumps(payload),
-        headers={"Content-Type": "application/json"},
+        headers=headers,
         access_token=access_token,
         **kwargs,
     )
